# -*- python -*-
# $RCSfile: quit.py,v $
# $Revision: 1.33 $
# $Author: vrc $
# $Date: 2007-09-26 22:02:30 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from ooflib.common import debug
from ooflib.common import mainthread
from ooflib.common import subthread
from ooflib.common.IO import mainmenu
from ooflib.common.IO import reporter
import ooflib.common.quit
import sys

# There's no code in here that actually addresses the GUI directly.
# This routine should work in non-GUI mode too.  Should it be moved to
# common/quit.py?  The code currently in common/quit.py differs from
# this code only in that common/quit.py doesn't give the user a chance
# to cancel the quitting procedure.  Presumably in non-GUI mode it's
# less easy to quit by mistake.

def quitQuery():
    if not ooflib.common.quit.quiet():
        if mainmenu.OOF.logChanged():
            answer = reporter.query("Save log file before quitting?",
                                    "Save", "Don't Save", "Cancel",
                                    default="Don't Save")
            if answer=="Cancel" or answer is None:
                return 0
            if answer=="Save":
                mainmenu.OOF.File.Save.Python_Log.invokeGUICallback()
                mainmenu.cleanlog()
##        else:
##            answer = reporter.query("Quitting so soon?", "Yes", "No",
##                                    default="Yes")
##            if answer == "No":
##                return 0
    return 1

# OOFMenu gui callback for the Quit commands.  Installed by
# mainmenuGUI.py and subWindow.py.
def quit(menuitem):
    if quitQuery():
        gui_cleanup()


####################################################################
#
# Over-ride cleanup, to do fancier things with the lock.
#

# The point of this is to block on a *subthread* while waiting for the
# quit lock, and then once you have it, transfer control back to the
# main thread for the actual cleanup.
def gui_quit_subthread():
    ## The order of execution here is a bit subtle.  We wait on the
    ## subthread until we have acquired the quitlock.  Then we install
    ## quit.cleanup as an idle callback on the main thread, and then
    ## release the lock.  The first thing that quit.cleanup does is to
    ## acquire the lock, so the main thread will block until the lock
    ## is released here.
    ooflib.common.quit.quitlock.acquire()

    try:
        mainthread.run(ooflib.common.quit.cleanup)

    finally:
        ooflib.common.quit.quitlock.release()
    
def gui_cleanup():
    subthread.execute_immortal(gui_quit_subthread)
    
