// -*- C++ -*-
// $RCSfile: color.h,v $
// $Revision: 1.20 $
// $Author: langer $
// $Date: 2008-02-26 22:16:55 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef COLORPROP_H
#define COLORPROP_H

#include "engine/property.h"
#include "common/ccolor.h"
#include <string>

// Color is slightly more general than just a gray value -- for now,
// it can be initialized to either (r,g,b), or to a gray level, 
// all of which are assumed to be between 0.0 and 1.0.
//   Since virtually all of the graphics is done in Python, this 
// class functions primarily as a place-holder, allowing materials
// to have color properties.  Before any graphics color assignments
// are contemplated, this color should be converted to the 
// appropriate GUI color object, thereby preventing a lot of 
// color callbacks through SWIG.

class ColorProp : public Property {
private:
  CColor cvalue_;
public:
  ColorProp(PyObject *reg, const std::string &nm, double g);
  ColorProp(PyObject *reg, const std::string &nm, 
	    double r, double g, double b);
  ColorProp(PyObject *reg, const std::string &nm, CColor &c);
  ColorProp(PyObject *reg, const std::string &nm, CColor *c);
  virtual ~ColorProp() {}
  const CColor &color() const { return cvalue_; }
  
  // TODO: These methods don't make sense for a ColorProp, but are
  // required by the Property base class. ColorProp should have a
  // different base class from the actual mathematically involved
  // properties.  See OrientationProp as well.
  virtual bool is_symmetric(const CSubProblem*) const { return true; }
  virtual bool constant_in_space() const { return true; }
  virtual int integration_order(const CSubProblem*, const Element*) const {
    return 0; }
};

#endif // COLORPROP_H
